import json
import os
import sys
import time

from collections import Counter
from operator import itemgetter, attrgetter



class TestCase:
    def __init__(self, result, testcase, error):
        self.result = result
        self.testcase = testcase
        self.error = error

    def to_dict(self):
        out = dict(name=self.testcase.name,
                   filename=self.testcase.filename,
                   linenr=self.testcase.linenr,
                   result=self.result)

        if self.error:
            out['error'] = dict(message=self.error.message,
                                traceback=self.error.traceback)

        return out


class Module:
    def __init__(self, name):
        self.name = name
        self.testcases = []

    def to_dict(self):

        # count the number succeeded, failed or error test cases
        result_counts = Counter(map(attrgetter('result'), self.testcases))

        # convert testcases to dicts
        testcases = list(map(lambda tc: tc.to_dict(), self.testcases))

        return dict(name=self.name,
                    nr_success=result_counts['success'],
                    nr_failed=result_counts['failed'],
                    nr_error=result_counts['error'],
                    testcases=testcases)


class Syntax:
    def __init__(self, error):
        self.filename = error.filename
        self.linenr = error.linenr
        self.column = error.column
        self.message = error.message

    def to_dict(self):
        tb = [dict(filename=self.filename,
                   linenr=self.linenr,
                   column=self.column)]
        out = dict(name=self.filename,
                   error=dict(message=self.message, traceback=tb))
        return out


class Sink:
    def __init__(self):
        self.records = []
        self.syntaxerrors = []
        self.output_file = 'nosetests.json'

    def write(self):
        output = self.generate()
        with open(self.output_file, 'w') as f:
            json.dump(output, f)

    def add(self, result, testcase, error):
        self.records.append(TestCase(result, testcase, error))

    def add_syntaxerror(self, error):
        self.syntaxerrors.append(Syntax(error))

    def generate(self):

        modules = {}

        for record in self.records:
            module_name = record.testcase.module

            if module_name not in modules:
                modules[module_name] = Module(module_name)

            module = modules[module_name]

            module.testcases.append(record)

        # Convert all modules to dicts
        modules_out = list(map(lambda m: m.to_dict(), modules.values()))
        modules_out.sort(key=itemgetter('name'))

        # Convert all syntax errors to dicts
        syntaxerrors = [error.to_dict() for error in self.syntaxerrors]

        return dict(metadata=self.metadata(),
                    syntaxerrors=syntaxerrors,
                    modules=modules_out)

    def metadata(self):
        return dict(command=' '.join(sys.argv),
                    time=time.time(),
                    cwd=os.getcwd())




from __future__ import absolute_import

import traceback
from collections import namedtuple
from nose.plugins import Plugin
from multiprocessing import Manager

from nosetests_json_extended_parallel.sink import Sink


class JsonExtendedPlugin(Plugin):
    name = 'json-extended-parallel'
    score = 2000

    def options(self, parser, env):
        Plugin.options(self, parser, env)

    def configure(self, options, config):
        Plugin.configure(self, options, config)

        self.config = config
        if not self.enabled:
            return

        self._sink = Sink()
        if not hasattr(self.config, '_nose_json_extended_state_'):
            manager = Manager()
            self._sink.records = manager.list()
            self.config._nose_json_extended_state_ = self._sink.records
        else:
            self._sink.records = self.config._nose_json_extended_state_

    def report(self, stream):
        self._sink.write()

    def addError(self, test, err, capt=None):
        if issubclass(err[0], SyntaxError):
            self._sink.add_syntaxerror(syntax_error_report(err[1]))
        else:
            self._sink.add('error',
                           testcase_description(test),
                           error_report(err))

    def addFailure(self, test, err, capt=None, tb_info=None):
        self._sink.add('failed',
                       testcase_description(test),
                       error_report(err))

    def addSuccess(self, test, capt=None):
        self._sink.add('success', testcase_description(test), None)


TestCaseDescription = namedtuple('TestCaseDescription',
                                 'module name filename linenr')


def testcase_description(test):

    module, name = _split_id(test.id())

    filename = None
    linenr = None

    try:
        filename = test.address()[0]
    except:
        pass

    try:
        methodname = test.test._testMethodName
        func = getattr(test.test, methodname)
        linenr = func.__func__.__code__.co_firstlineno
    except:
        pass

    return TestCaseDescription(module, name, filename, linenr)


ErrorReport = namedtuple('ErrorReport', 'message traceback')


def error_report(err):
    message_list = traceback.format_exception_only(err[0], err[1])
    message = '\n'.join(message_list).strip('\n')

    tb = list(wrap_traceback(traceback.extract_tb(err[2])))

    return ErrorReport(message, tb)


def wrap_traceback(traceback_in):
    names = ('filename', 'linenr', 'function', 'line')
    for tb in traceback_in:
        yield dict(zip(names, tb))


def _split_id(test_id):
    parts = test_id.split('.')
    return '.'.join(parts[:-1]), parts[-1]




SyntaxErrorReport = namedtuple('SyntaxErrorReport', 'filename linenr column message')

def syntax_error_report(error):
    filename = error.filename
    linenr = error.lineno
    column = error.offset

    message = error.__class__.__name__ + ': ' + str(error)

    if error.text:
        message += '\n\n' + error.text.strip()

        if error.offset:
            text = error.text.rstrip()
            nr_white_space = len(text) - len(text.lstrip())
            nr = error.offset-nr_white_space-1
            message += '\n' + ' ' * nr + '^'

    return SyntaxErrorReport(filename, linenr, column, message)



from __future__ import absolute_import

import unittest

from nosetests_json_extended_parallel.sink import Sink
from nosetests_json_extended_parallel.plugin import TestCaseDescription as TC
from nosetests_json_extended_parallel.plugin import SyntaxErrorReport
from nosetests_json_extended_parallel.plugin import ErrorReport


class SinkTest(unittest.TestCase):

    def test_module_headers(self):
        sink = Sink()
        sink.add('success', TC('module.A', 'test1', None, None), None)
        sink.add('success', TC('module.A', 'test2', None, None), None)
        sink.add('success', TC('module.B', 'test3', None, None), None)
        sink.add('failed', TC('module.B', 'test4', None, None), None)
        sink.add('success', TC('module.C', 'test5', None, None), None)
        sink.add('error', TC('module.C', 'test6', None, None), None)
        sink.add_syntaxerror(SyntaxErrorReport('file.py', 3, 4, 'msg'))

        out = sink.generate()

        self.assertEqual(out['modules'][0]['name'], 'module.A')
        self.assertEqual(out['modules'][0]['nr_success'], 2)
        self.assertEqual(out['modules'][0]['nr_failed'], 0)
        self.assertEqual(out['modules'][0]['nr_error'], 0)

        self.assertEqual(out['modules'][1]['name'], 'module.B')
        self.assertEqual(out['modules'][1]['nr_success'], 1)
        self.assertEqual(out['modules'][1]['nr_failed'], 1)
        self.assertEqual(out['modules'][1]['nr_error'], 0)

        self.assertEqual(out['modules'][2]['name'], 'module.C')
        self.assertEqual(out['modules'][2]['nr_success'], 1)
        self.assertEqual(out['modules'][2]['nr_failed'], 0)
        self.assertEqual(out['modules'][2]['nr_error'], 1)

    def _single_testcase(self, *a):
        sink = Sink()
        sink.add(*a)
        return sink.generate()

    def test_testcase_success(self):
        tc = TC('mod', 'test1', None, None)
        out = self._single_testcase('success', tc, None)
        tc1 = out['modules'][0]['testcases'][0]
        self.assertEqual(tc1['name'], 'test1')
        self.assertEqual(tc1['result'], 'success')

    def test_testcase_failed(self):
        er = ErrorReport('error_desc', ['tb0', 'tb1'])
        tc = TC('mod', 'test1', None, None)
        out = self._single_testcase('error', tc, er)
        tc1 = out['modules'][0]['testcases'][0]

        self.assertEqual(tc1['name'], 'test1')
        self.assertEqual(tc1['result'], 'error')
        self.assertEqual(tc1['error']['message'], 'error_desc')
        self.assertEqual(tc1['error']['traceback'], ['tb0', 'tb1'])

    def test_testcase_syntax(self):
        sink = Sink()
        sink.add_syntaxerror(SyntaxErrorReport('filename', 6, 18, 'message'))
        out = sink.generate()

        expected = dict(name='filename',
                        error=dict(message='message',
                                   traceback=[dict(filename='filename',
                                                   linenr=6,
                                                   column=18)]))

        self.assertEqual(out['syntaxerrors'], [expected])


class SyntaxTest(unittest.TestCase):

    def test_module_headers(self):
        pass



from __future__ import absolute_import

import unittest
from nose.plugins import PluginTester
from nosetests_json_extended_parallel.plugin import (JsonExtendedPlugin, wrap_traceback,
                                            syntax_error_report)


class Helper(PluginTester, unittest.TestCase):
    activate = '--with-json-extended-parallel'
    plugins = [JsonExtendedPlugin()]

    def makeSuite(self):
        class Stub(unittest.TestCase):
            def runTest(tc):
                self.stubtest(tc)

        return [Stub('runTest')]

    @property
    def sink(self):
        return self.plugins[0]._sink


class SucceedsTest(Helper):

    def stubtest(self, tc):
        tc.assertTrue(True)

    def test(self):
        result = self.sink.records[0]

        self.assertEquals(result.result, 'success')
        self.assertEquals(result.testcase.module,
                          'nosetests_json_extended_parallel.test_plugin.Stub')
        self.assertEquals(result.testcase.name, 'runTest')
        self.assertEquals(result.error, None)
        self.assertEquals(result.testcase.filename[-14:], 'test_plugin.py')
        self.assertFalse(result.testcase.linenr is None)


class FailureTest(Helper):

    def stubtest(self, tc):
        tc.assertTrue(False)

    def test(self):
        result = self.sink.records[0]
        self.assertEquals(result.result, 'failed')
        message = 'AssertionError: False is not true'
        self.assertEquals(result.error.message, message)
        self.assertIsInstance(result.error.traceback, list)


class ErrorTest(Helper):

    def stubtest(self, tc):
        raise Exception('errormessage')

    def test(self):
        result = self.sink.records[0]
        self.assertEquals(result.result, 'error')
        self.assertEquals(result.error.message, 'Exception: errormessage')
        self.assertIsInstance(result.error.traceback, list)


class SyntaxErrorTest(Helper):

    def stubtest(self, tc):
        raise SyntaxError('errormessage')

    def test(self):
        result = self.sink.syntaxerrors[0]
        self.assertEquals(result.filename, None)
        self.assertEquals(result.linenr, None)
        self.assertEquals(result.column, None)
        self.assertEquals(result.message, 'SyntaxError: errormessage')


class TracebackWrapperTest(unittest.TestCase):

    def test(self):
        out = list(wrap_traceback([('file.py', 4, 'foo', 'bar()')]))

        self.assertEquals(out[0], dict(filename='file.py',
                                       linenr=4,
                                       function='foo',
                                       line='bar()'))


class SyntaxErrorReportTest(unittest.TestCase):

    def test(self):

        params = ('filename', 6, 18, '    def func(self)\n')
        report = syntax_error_report(SyntaxError('message', params))

        message = 'SyntaxError: message (filename, line 6)\n\n' \
                  'def func(self)\n             ^'

        self.assertEqual(report.filename, 'filename')
        self.assertEqual(report.linenr, 6)
        self.assertEqual(report.column, 18)
        self.assertEqual(report.message, message)



#!/usr/bin/env python

from setuptools import setup

description = 'Create json logging output for python' + \
              'nosetests unittest framework',

setup(
    name='nosetests-json-extended-parallel',
    version='1.0',
    author='Rui Li',
    author_email='rui.li.spam@gmail.com',
    description=description,
    url='https://github.com/ruivapps/nosetests-json-extended-parallel',
    download_url='https://github.com/ruivapps/nosetests-json-extended-parallel/tarball/1.0',
    packages=['nosetests_json_extended_parallel'],
    zip_safe=False,
    entry_points={
        'nose.plugins.0.10': [
            'nosetests_json_extended_parallel = ' +
            'nosetests_json_extended_parallel.plugin:JsonExtendedPlugin'
        ]
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Testing'
    ],
    long_description=open('README.rst', 'r').read()
)
